﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.Xml.Serialization;
using System.IO;
using System.Security.Cryptography.X509Certificates;
using System.Security.Cryptography;
using System.Security.Cryptography.Xml;

namespace NTK.Demos
{
    partial class DocumentCreator
    {
        internal static XmlElement OM_v1()
        {
            Person p = new Person(1);
            p.Name = "Matevž";
            p.Surname = "Gačnik";
            return p.Serialize();
        }

        internal static XmlElement OM_v2()
        {
            Person p = new Person(2);
            p.Name = "Matevž";
            p.Surname = "Gačnik";
            p.Age = 31;
            return p.Serialize();
        }

        internal static XmlElement OM_v3()
        {
            Person p = new Person();
            p.Name = "Matevž";
            p.Surname = "Gačnik";
            p.Age = 31;
            X509Certificate2 cert = SelectCert();
            p.Certificate = cert.SerialNumber;
            return p.Serialize(cert);
        }


        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        
        private static X509Certificate2 SelectCert()
        {
            X509Store store = new X509Store(StoreName.My, StoreLocation.CurrentUser);
            store.Open(OpenFlags.ReadOnly | OpenFlags.OpenExistingOnly);
            X509Certificate2Collection collection = (X509Certificate2Collection)store.Certificates;
            X509Certificate2Collection fcollection = (X509Certificate2Collection)collection.Find(X509FindType.FindByTimeValid, DateTime.Now, false);
            X509Certificate2Collection scollection = X509Certificate2UI.SelectFromCollection(fcollection, "Select Signing Certificate", "Select a certificate from the following list to get information on that certificate", X509SelectionFlag.SingleSelection);
            return scollection[0];
        }
    }

    internal class Person
    {
        const int maxVersion = 3;
        int version;
        string name;
        string surname;
        int age;
        string cert;

        Person_v1 p1;
        Person_v2 p2;
        Person_v3 p3;

        public Person() : this(maxVersion) { }

        public Person(int version)
        {
            this.version = version;
            if (version == 1)
                p1 = new Person_v1();
            if (version == 2)
                p2 = new Person_v2();
            if (version == 3)
                p3 = new Person_v3();
            if (version > 3)
                throw new ApplicationException("Version not supported.");
        }

        public int Version
        {
            get { return version; }
        }

        public string Name
        {
            get { return name; }
            set { name = value; }
        }

        public string Surname
        {
            get { return surname; }
            set { surname = value; }
        }

        public int Age
        {
            get { return age; }
            set { age = value; }
        }

        public string Certificate
        {
            get { return cert; }
            set { cert = value; }
        }

        public XmlElement Serialize()
        {
            return PerformSerialization(false, null);
        }

        public XmlElement Serialize(X509Certificate2 signingCert)
        {
            return PerformSerialization(true, signingCert);
        }

        private XmlElement PerformSerialization(bool sign, X509Certificate2 signingCert)
        {
            // check validity
            if ((name == null) || (name.Length == 0))
                throw new Exception("Name is a required field.");
            if ((surname == null) || (surname.Length == 0))
                throw new Exception("Surname is a required field.");
            if ((version >= 2) && (age < 1))
                throw new Exception("Age is a required field.");
            if ((version >= 3) && ((cert == null) || (cert.Length == 0)))
                throw new Exception("Certificate is a required field.");

            XmlSerializer ser;
            MemoryStream stream = new MemoryStream();
            switch (version)
            {
                case 1:
                    ser = new XmlSerializer(typeof(Person_v1));
                    // hydrate data
                    p1.Name = this.name;
                    p1.Surname = this.surname;
                    ser.Serialize(stream, p1);
                    break;
                case 2:
                    ser = new XmlSerializer(typeof(Person_v2));
                    // hydrate data
                    p2.Name = this.name;
                    p2.Surname = this.surname;
                    p2.Age = this.age;
                    ser.Serialize(stream, p2);
                    break;
                case 3:
                default:
                    ser = new XmlSerializer(typeof(Person_v3));
                    // hydrate data
                    p3.Name = this.name;
                    p3.Surname = this.surname;
                    p3.Age = this.age;
                    p3.Certificate = signingCert.SerialNumber;
                    ser.Serialize(stream, p3);
                    break;
            }

            stream.Position = 0;
            XmlDocument doc = new XmlDocument();
            doc.Load(stream);

            // sign the document
            if (sign)
            {
                try
                {
                    // get private key and import it into RSA object
                    RSACryptoServiceProvider key = (RSACryptoServiceProvider)signingCert.PrivateKey;

                    // store cert data in signature -> exponent+modulus and complete certificate, public key only
                    KeyInfo keyInfo = new KeyInfo();

                    // exponent + modulus
                    keyInfo.AddClause(new RSAKeyValue(key));

                    // complete certificate public key
                    keyInfo.AddClause(new KeyInfoX509Data(signingCert));

                    // create signed xml object
                    SignedXml signedXml = new SignedXml(doc);

                    // add keyinfo
                    signedXml.KeyInfo = keyInfo;
                    signedXml.SigningKey = key;

                    // add reference to signature
                    signedXml.AddReference(new Reference(""));

                    // compute signature
                    signedXml.ComputeSignature();

                    // get the XML representation of the signature
                    XmlElement xmlSignature = signedXml.GetXml();
                    XmlNode nodeSource = doc.ImportNode(xmlSignature, true);

                    // insert signature under appropriate PDoc data element
                    XmlNode nodeDest = doc.DocumentElement;
                    nodeDest.AppendChild(nodeSource);
                }
                catch (Exception ex)
                {
                    throw new ApplicationException("Problem during digital signing: " + ex.Message);
                }
            }
            return doc.DocumentElement;
        }
    }
}
