﻿using System;
using System.Collections.Generic;
using System.Xml;
using System.Text;
using System.Xml.Schema;
using System.IO;

namespace NTK.Demos
{
    public static class PersonValidator
    {
        private static bool isValid;

        internal static int GetSchemaVersion(XmlDocument person)
        {
            XmlNamespaceManager xmlns = new XmlNamespaceManager(person.NameTable);
            xmlns.AddNamespace("p1", "http://schemas.gama-system.com/person_v1");
            xmlns.AddNamespace("p2", "http://schemas.gama-system.com/person_v2");
            xmlns.AddNamespace("p3", "http://schemas.gama-system.com/person_v3");
            XmlNode node = person.SelectSingleNode("/p1:Person/@version | /p2:Person/@version | /p3:Person/@version", xmlns);
            return Convert.ToInt32(node.Value);
        }

        internal static string GetNameSurname(XmlDocument person)
        {
            XmlNamespaceManager xmlns = new XmlNamespaceManager(person.NameTable);
            xmlns.AddNamespace("p1", "http://schemas.gama-system.com/person_v1");
            xmlns.AddNamespace("p2", "http://schemas.gama-system.com/person_v2");
            xmlns.AddNamespace("p3", "http://schemas.gama-system.com/person_v3");
            XmlNode name = person.SelectSingleNode("/p1:Person/p1:Name | /p2:Person/p2:Name | /p3:Person/p3:Name", xmlns);
            XmlNode surname = person.SelectSingleNode("/p1:Person/p1:Surname | /p2:Person/p2:Surname | /p3:Person/p3:Surname", xmlns);
            XmlNode age = person.SelectSingleNode("/p2:Person/p2:Age | /p3:Person/p3:Age", xmlns);
            if (age == null)
                return String.Format("{0} {1}", name.InnerText, surname.InnerText);
            else
                return String.Format("{0} {1} ({2})", name.InnerText, surname.InnerText, age.InnerText);
        }

        internal static bool IsRepresentationValid(XmlDocument person)
        {
            isValid = true;

            // add all available schemas
            XmlReaderSettings settingsSchema = new XmlReaderSettings();
            settingsSchema.ValidationType = ValidationType.None; 
            XmlSchemaSet schemaSet = new XmlSchemaSet();
            string[] schemas = Directory.GetFiles("Schemas", "*.xsd", SearchOption.TopDirectoryOnly);
            for (int i = 0; i < schemas.Length; i++)
            {
                XmlReader xr = XmlReader.Create(schemas[i], settingsSchema);
                schemaSet.Add(null, xr);
            }

            // set up validation
            XmlReaderSettings settingsDoc = new XmlReaderSettings();
            settingsDoc.ValidationType = ValidationType.Schema;
            settingsDoc.Schemas = schemaSet;
            settingsDoc.ValidationEventHandler += new ValidationEventHandler(ValidationEventHandler);
            settingsDoc.ValidationFlags = XmlSchemaValidationFlags.ReportValidationWarnings;

            // get document
            XmlReader xrDoc = XmlReader.Create(new StringReader(person.OuterXml), settingsDoc);

            // validate document
            while (xrDoc.Read());
            
            return isValid;
        }

        private static void ValidationEventHandler(object sender, ValidationEventArgs e)
        {
            if (e.Severity == XmlSeverityType.Error) isValid = false;
        }
    }
}
